/*
 * file CqUserDb.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqUserDb
 *
 *  Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.wvcm.Feedback;
import javax.wvcm.Location;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProperty;
import com.ibm.rational.wvcm.stp.StpRepository;
import com.ibm.rational.wvcm.stpex.StpExBase;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;
import com.ibm.rational.wvcm.stpex.StpExBase.NameException;


/**
 * A proxy for a ClearQuest user database
 * <p>
 * In addition to representing the physical database, a CqUserDb object also
 * represents the on-going ClearQuest session between the user and that
 * database. As such, it represents a user-specific, server-resident area in
 * which changes to CqContextResources are performed. This server-area is
 * referred to as the database's <i>change context</i>. Here, the term
 * <i>change</i> refers to the creation, revision, or deletion of a resource,
 * its content or its properties as a consequence of some server interaction.
 * <p>
 * In general, any changes made to a CqContextResource are held in the change
 * context area until the client explicitly directs the server to deliver those
 * changes to the database. Each operation that changes a database object has a
 * parameter named <code>deliveryOrder</code> (usually the last parameter).
 * This parameter controls the disposition of the changes after the operation
 * completes. The parameter is a List<CqContextResource> and specifies the
 * resources (in the change context) to be delivered to the database and the
 * order in which those resources are to be delivered.
 * <p>
 * If the operation fails, no delivery is attempted.
 * <p>
 * Special instances of List<CqContextResource> are defined in the CqProvider
 * interface as short-cuts for specifying the resources to be delivered. See the
 * documentation for {@link CqProvider#HOLD}, {@link CqProvider#AUTO},
 * {@link CqProvider#DELIVER}, and {@link CqProvider#DELIVER_ALL} for more
 * information.
 * <p>
 * The properties that describe the state of the database's change context are
 * {@link #CONTEXT_IS_EMPTY}, {@link #MODIFIED_RESOURCES_LIST}, and
 * {@link #MORIBUND_RESOURCES_LIST}.
 * <p>
 * In addition to the operations that modify a CqContextResource, the following
 * methods are defined by CqUserDb to aid in the management of the database's
 * change context: {@link #doClearContext(Feedback)},
 * {@link #doDeliver(Feedback, List)}, and {@link #doRevert(Feedback, List)}.
 * <p>
 * The user-friendly specification for the location of a user database has the
 * form
 * 
 * <pre>
 *  <b>cq.userdb:</b><i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 * 
 * @see CqContextResource
 */
public interface CqUserDb extends StpRepository, CqDb
{
    /**
     * Answers whether or not there are modified or moribund resources still in
     * the change context associated with this user database.
     */
    PropertyName<Boolean> CONTEXT_IS_EMPTY =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "context-is-empty");

    /**
     * Returns the value of the {@link #CONTEXT_IS_EMPTY CONTEXT_IS_EMPTY}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the change context has undelivered resources;
     *         <b>false</b> if there are no modified or moribund resources in
     *         the change context.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CONTEXT_IS_EMPTY CONTEXT_IS_EMPTY} property.
     */
    boolean getContextIsEmpty() throws WvcmException;

    /**
     * A list of the modified resources currently in the database's change
     * context. The changes made to these resources will not be seen until the
     * changes are delivered.
     */
    PropertyName<ResourceList<CqContextResource>> MODIFIED_RESOURCES_LIST =
        new PropertyName<ResourceList<CqContextResource>>(PROPERTY_NAMESPACE,
                                                          "modified-resources-list");

    /**
     * Returns the value of the
     * {@link #MODIFIED_RESOURCES_LIST MODIFIED_RESOURCES_LIST} property as
     * defined by this proxy.
     * 
     * @return A list of the context resources of the database that have been
     *         modified but not yet committed to the database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #MODIFIED_RESOURCES_LIST MODIFIED_RESOURCES_LIST}
     *             property.
     */
    ResourceList<CqContextResource> getModifiedResourcesList()
        throws WvcmException;

    /**
     * A list of the context resources whose deletion has been initiated but not
     * yet committed. The deletion will not be visible until these resources are
     * delivered to the database. The deletion of a moribund resource can be
     * canceled using the doRevert() method.
     */
    PropertyName<ResourceList<CqContextResource>> MORIBUND_RESOURCES_LIST =
        new PropertyName<ResourceList<CqContextResource>>(PROPERTY_NAMESPACE,
                                                          "moribund-resources-list");

    /**
     * Returns the value of the
     * {@link #MORIBUND_RESOURCES_LIST MORIBUND_RESOURCES_LIST} property as
     * defined by this proxy.
     * 
     * @return A list of the context resources of the database whose deletion
     *         has been initiated but not yet committed to the database
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #MORIBUND_RESOURCES_LIST MORIBUND_RESOURCES_LIST}
     *             property.
     */
    ResourceList<CqContextResource> getMoribundResourcesList()
        throws WvcmException;

    /**
     * The first-level query folder items in this user database. At this level
     * are just two query folders containing, respectively, the public and
     * personal query folder items.
     */
    PropertyName<ResourceList<CqQueryFolderItem>> QUERY_FOLDER_ITEMS =
        new PropertyName<ResourceList<CqQueryFolderItem>>(PROPERTY_NAMESPACE,
                                                          "query-folder-items");

    /**
     * Returns the value of the {@link #QUERY_FOLDER_ITEMS QUERY_FOLDER_ITEMS}
     * property as defined by this proxy.
     * 
     * @return A list of two CqQueryFolder proxies; one for the public queries
     *         folder and one for the personal queries folder.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #QUERY_FOLDER_ITEMS QUERY_FOLDER_ITEMS} property.
     */
    ResourceList<CqQueryFolderItem> getQueryFolderItems() throws WvcmException;

    /**
     * The CqRecordType that the schema suggests should be used by default when
     * creating records and when finding records by id.
     */
    PropertyName<CqRecordType> DEFAULT_RECORD_TYPE =
        new PropertyName<CqRecordType>(PROPERTY_NAMESPACE,
                                       "default-record-type");

    /**
     * Returns the value of the {@link #DEFAULT_RECORD_TYPE DEFAULT_RECORD_TYPE}
     * property as defined by this proxy.
     * 
     * @return A CqRecordType proxy for the default record type defined for this
     *         user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DEFAULT_RECORD_TYPE DEFAULT_RECORD_TYPE} property.
     */
    CqRecordType getDefaultRecordType() throws WvcmException;

    /** A list of the record types defined by this user database. */
    PropertyName<ResourceList<CqRecordType>> RECORD_TYPE_SET =
        new PropertyName<ResourceList<CqRecordType>>(PROPERTY_NAMESPACE,
                                                     "record-type-set");

    /**
     * Returns the value of the {@link #RECORD_TYPE_SET RECORD_TYPE_SET}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a CqRecordType proxy for each record
     *         type defined by this user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #RECORD_TYPE_SET RECORD_TYPE_SET} property.
     */
    ResourceList<CqRecordType> getRecordTypeSet() throws WvcmException;

    /**
     * The query folder containing all queries in this database privately owned
     * by the current user.
     */
    PropertyName<CqQueryFolder> PERSONAL_FOLDER =
        new PropertyName<CqQueryFolder>(PROPERTY_NAMESPACE, "personal-folder");

    /**
     * Returns the value of the {@link #PERSONAL_FOLDER PERSONAL_FOLDER}
     * property as defined by this proxy.
     * 
     * @return A CqQueryFolder proxy for the root folder containing all personal
     *         queries.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PERSONAL_FOLDER PERSONAL_FOLDER} property.
     */
    CqQueryFolder getPersonalFolder() throws WvcmException;

    /** The query folder containing all public queries in this database */
    PropertyName<CqQueryFolder> PUBLIC_FOLDER =
        new PropertyName<CqQueryFolder>(PROPERTY_NAMESPACE, "public-folder");

    /**
     * Returns the value of the {@link #PUBLIC_FOLDER PUBLIC_FOLDER} property as
     * defined by this proxy.
     * 
     * @return A CqQueryFolder proxy for the root query folder that contains all
     *         public queries.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PUBLIC_FOLDER PUBLIC_FOLDER} property.
     */
    CqQueryFolder getPublicFolder() throws WvcmException;

    /**
     * All queries (no charts or reports) defined in this database and
     * accessible to the current user.
     */
    PropertyName<ResourceList<CqQuery>> ALL_QUERIES =
        new PropertyName<ResourceList<CqQuery>>(PROPERTY_NAMESPACE,
                                                "all-queries");

    /**
     * Returns the value of the {@link #ALL_QUERIES ALL_QUERIES} property as
     * defined by this proxy.
     * 
     * @return A ResourceList of CqQuery proxies containing all the queries of
     *         type CqQuery.QueryType.LIST defined in this user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_QUERIES ALL_QUERIES} property.
     */
    ResourceList<CqQuery> getAllQueries() throws WvcmException;

    /** The queries (no charts or reports) owned by the current user */
    PropertyName<ResourceList<CqQuery>> PERSONAL_QUERIES =
        new PropertyName<ResourceList<CqQuery>>(PROPERTY_NAMESPACE,
                                                "personal-queries");

    /**
     * Returns the value of the {@link #PERSONAL_QUERIES PERSONAL_QUERIES}
     * property as defined by this proxy.
     * 
     * @return A ResourceList of CqQuery proxies containing all the personal
     *         queries (no charts or reports) defined in this database for the
     *         current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PERSONAL_QUERIES PERSONAL_QUERIES} property.
     */
    ResourceList<CqQuery> getPersonalQueries() throws WvcmException;

    /**
     * The queries (no charts or reports) defined in the public folder of this
     * database
     */
    PropertyName<ResourceList<CqQuery>> PUBLIC_QUERIES =
        new PropertyName<ResourceList<CqQuery>>(PROPERTY_NAMESPACE,
                                                "public-queries");

    /**
     * Returns the value of the {@link #PUBLIC_QUERIES PUBLIC_QUERIES} property
     * as defined by this proxy.
     * 
     * @return A ResourceList of CqQuery proxies containing all of the public
     *         queries (no charts or reports) defined in this user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PUBLIC_QUERIES PUBLIC_QUERIES} property.
     */
    ResourceList<CqQuery> getPublicQueries() throws WvcmException;

    /**
     * All chart queries defined in this database and accessible to the current
     * user.
     */
    PropertyName<ResourceList<CqQuery>> ALL_CHARTS =
        new PropertyName<ResourceList<CqQuery>>(PROPERTY_NAMESPACE,
                                                "all-charts");

    /**
     * Returns the value of the {@link #ALL_CHARTS ALL_CHARTS} property as
     * defined by this proxy.
     * 
     * @return A ResourceList of CqQuery proxies containing all of the chart
     *         queries accessible to the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_CHARTS ALL_CHARTS} property.
     */
    ResourceList<CqQuery> getAllCharts() throws WvcmException;

    /** All chart queries owned by the current user. */
    PropertyName<ResourceList<CqQuery>> PERSONAL_CHARTS =
        new PropertyName<ResourceList<CqQuery>>(PROPERTY_NAMESPACE,
                                                "personal-charts");

    /**
     * Returns the value of the {@link #PERSONAL_CHARTS PERSONAL_CHARTS}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a CqQuery proxy for each chart query
     *         owned by the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PERSONAL_CHARTS PERSONAL_CHARTS} property.
     */
    ResourceList<CqQuery> getPersonalCharts() throws WvcmException;

    /** The chart queries defined in this database */
    PropertyName<ResourceList<CqQuery>> PUBLIC_CHARTS =
        new PropertyName<ResourceList<CqQuery>>(PROPERTY_NAMESPACE,
                                                "public-charts");

    /**
     * Returns the value of the {@link #PUBLIC_CHARTS PUBLIC_CHARTS} property as
     * defined by this proxy.
     * 
     * @return A ResourceList containing a CqQuery proxy for each chart query in
     *         this user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PUBLIC_CHARTS PUBLIC_CHARTS} property.
     */
    ResourceList<CqQuery> getPublicCharts() throws WvcmException;

    /**
     * All report queries defined in this database and accessible to the current
     * user.
     */
    PropertyName<ResourceList<CqReport>> ALL_REPORTS =
        new PropertyName<ResourceList<CqReport>>(PROPERTY_NAMESPACE,
                                                 "all-reports");

    /**
     * Returns the value of the {@link #ALL_REPORTS ALL_REPORTS} property as
     * defined by this proxy.
     * 
     * @return A ResourceList containing a CqQuery proxy for each public report
     *         query and each report query owned by the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_REPORTS ALL_REPORTS} property.
     */
    ResourceList<CqReport> getAllReports() throws WvcmException;

    /** All report queries owned by the current user. */
    PropertyName<ResourceList<CqReport>> PERSONAL_REPORTS =
        new PropertyName<ResourceList<CqReport>>(PROPERTY_NAMESPACE,
                                                 "personal-reports");

    /**
     * Returns the value of the {@link #PERSONAL_REPORTS PERSONAL_REPORTS}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a CqQuery proxy for each report query
     *         owned by the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PERSONAL_REPORTS PERSONAL_REPORTS} property.
     */
    ResourceList<CqReport> getPersonalReports() throws WvcmException;

    /**
     * All reports defined in this database that are accessible to the current
     * user
     */
    PropertyName<ResourceList<CqReport>> PUBLIC_REPORTS =
        new PropertyName<ResourceList<CqReport>>(PROPERTY_NAMESPACE,
                                                 "public-reports");

    /**
     * Returns the value of the {@link #PUBLIC_REPORTS PUBLIC_REPORTS} property
     * as defined by this proxy.
     * 
     * @return A ResourceList containing a CqQuery proxy for each report query
     *         accessible to the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PUBLIC_REPORTS PUBLIC_REPORTS} property.
     */
    ResourceList<CqReport> getPublicReports() throws WvcmException;

    /** The database set that contains this user database */
    PropertyName<CqDbSet> DB_SET =
        new PropertyName<CqDbSet>(PROPERTY_NAMESPACE, "db-set");

    /**
     * Returns the value of the {@link #DB_SET DB_SET} property as defined by
     * this proxy.
     * 
     * @return A CqDbSet proxy representing the database set that contains this
     *         user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DB_SET DB_SET} property.
     */
    CqDbSet getDbSet() throws WvcmException;

    /**
     * The interval at which to check for user timeouts.
     * 
     * <p>
     * ClearQuest uses this property to determine how often it should check the
     * status of user connections. When the specified interval lapses,
     * ClearQuest checks each user connection for activity. If no activity has
     * been detected recently, ClearQuest checks the TimeoutInterval property to
     * see if the user's connection has timed out.
     */
    PropertyName<Long> CHECK_TIMEOUT_INTERVAL =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "check-timeout-interval");

    /**
     * Returns the value of the {@link #CHECK_TIMEOUT_INTERVAL CHECK_TIME
     * OUT_INTERVAL} property as defined by this proxy.
     * 
     * @return A long value representing the CHECK_TIMEOUT_INTERVAL in
     *         milliseconds
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CHECK_TIMEOUT_INTERVAL CHECK_TIMEOUT_INTERVAL}
     *             property.
     */
    long getCheckTimeoutInterval() throws WvcmException;

    /**
     * The name by which the database vendor installation knows this user
     * database.
     */
    PropertyName<String> DATABASE_NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "database-name");

    /**
     * Returns the value of the {@link #DATABASE_NAME DATABASE_NAME} property as
     * defined by this proxy.
     * 
     * @return A String containing the name by which the database vendor
     *         installation knows this user database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DATABASE_NAME DATABASE_NAME} property.
     */
    String getDatabaseName() throws WvcmException;

    /** A list of the dynamic choice lists that are defined in this database */
    PropertyName<ResourceList<CqDynamicChoiceList>> DYNAMIC_CHOICE_LISTS =
        new PropertyName<ResourceList<CqDynamicChoiceList>>(PROPERTY_NAMESPACE,
                                                            "dynamic-choice-lists");

    /**
     * Returns the value of the {@link #DYNAMIC_CHOICE_LISTS} property as
     * defined by this proxy.
     * 
     * @return A ResourceList containing a CqDynamicChoiceList proxy for each
     *         dynamic choice list defined in this database. Will never be
     *         <b>null</b>, but may be empty.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DYNAMIC_CHOICE_LISTS} property.
     */
    ResourceList<CqDynamicChoiceList> getDynamicChoiceLists()
        throws WvcmException;

    /**
     * The logical database name. The name to use in an StpLocation to identify
     * this database; same as DISPLAY_NAME.
     */
    PropertyName<String> NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "name");

    /**
     * Returns the value of the {@link #NAME NAME} property as defined by this
     * proxy.
     * 
     * @return A String containing the logical name of this database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #NAME NAME} property.
     */
    String getName() throws WvcmException;

    /**
     * The schema revision currently in use by the database.
     * <p>
     * This is a read-only property; it can be viewed but not set.
     * <p>
     * To change the schema revision of an existing database, you must upgrade
     * the database by calling the Upgrade method. If you are creating a new
     * database, you can set its initial schema revision using the
     * SetInitialSchemaRev method.
     */
    PropertyName<Long> SCHEMA_REV =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "schema-rev");

    /**
     * Returns the value of the {@link #SCHEMA_REV SCHEMA_REV} property as
     * defined by this proxy.
     * 
     * @return A Long containing the schema revision number.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SCHEMA_REV SCHEMA_REV} property.
     */
    long getSchemaRev() throws WvcmException;

    /** The name of the server on which this database resides. */
    PropertyName<String> SERVER =
        new PropertyName<String>(PROPERTY_NAMESPACE, "server");

    /**
     * Returns the value of the {@link #SERVER SERVER} property as defined by
     * this proxy.
     * 
     * @return A String containing the name of the server on which this database
     *         is installed.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SERVER SERVER} property.
     */
    String getServer() throws WvcmException;

    /**
     * The login timeout interval.
     * 
     * <p>
     * ClearQuest periodically checks user connections and disconnects users who
     * have been idle for too long. If a user has been idle for a period of time
     * greater than the value in this property, ClearQuest disconnects the
     * user's session at the next tick of the CHECK_TIMEOUT_INTERVAL.
     */
    PropertyName<Long> TIMEOUT_INTERVAL =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "timeout-interval");

    /**
     * Returns the value of the {@link #TIMEOUT_INTERVAL TIMEOUT_INTERVAL}
     * property as defined by this proxy.
     * 
     * @return An integer indicating the TIMEOUT_INTERVAL in milliseconds
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #TIMEOUT_INTERVAL TIMEOUT_INTERVAL} property.
     */
    long getTimeoutInterval() throws WvcmException;

    /** the vendor type of the database */
    PropertyName<VendorType> VENDOR =
        new PropertyName<VendorType>(PROPERTY_NAMESPACE, "vendor");

    /**
     * An enumeration of the types of vendors of supported databases
     */
    enum VendorType implements StpExEnumeration
    {
        /** The VendorType for an SQL Server database */
        SQL_SERVER,

        /** The VendorType for an Microsoft Access database */
        MS_ACCESS,

        /** The VendorType for an SQL Anywhere database */
        SQL_ANYWHERE,

        /** The VendorType for an Oracle database */
        ORACLE,

        /** The VendorType for a DB2 database */
        DB2;
    }

    /**
     * Returns the value of the {@link #VENDOR VENDOR} property as defined by
     * this proxy.
     * 
     * @return A VendorType enumerator indicating vendor of the physical
     *         database.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #VENDOR VENDOR} property.
     */
    VendorType getVendor() throws WvcmException;

    /** The connection options for the physical database. */
    PropertyName<String> CONNECT_OPTIONS =
        new PropertyName<String>(PROPERTY_NAMESPACE, "connect-options");

    /**
     * Returns the value of the {@link #CONNECT_OPTIONS CONNECT_OPTIONS}
     * property as defined by this proxy.
     * 
     * @return A String containing the vendors connection options.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CONNECT_OPTIONS CONNECT_OPTIONS} property.
     */
    String getConnectOptions() throws WvcmException;

    /**
     * Implements CQSession.GetSessionFeatureLevel
     */
    PropertyName<Long> FEATURE_LEVEL =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "feature-level");

    /**
     * Returns the value of the {@link #FEATURE_LEVEL FEATURE_LEVEL} property as
     * defined by this proxy.
     * 
     * @return The value. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FEATURE_LEVEL FEATURE_LEVEL} property.
     */
    long getFeatureLevel() throws WvcmException;

    /**
     * Answers whether or not the current user has logged in as a restricted
     * user
     */
    PropertyName<Boolean> IS_RESTRICTED_USER =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-restricted-user");

    /**
     * Returns the value of the {@link #IS_RESTRICTED_USER IS_RESTRICTED_USER}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the current user logged in as a restricted user;
     *         <b>false</b> otherwise;.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_RESTRICTED_USER IS_RESTRICTED_USER} property.
     */
    boolean getIsRestrictedUser() throws WvcmException;

    /**
     * Answers whether or not the database has been activated for multisite
     * operations (even if the database is the only existing replica).
     */
    PropertyName<Boolean> IS_MULTISITE_ACTIVATED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-multisite-activated");

    /**
     * Returns the value of the
     * {@link #IS_MULTISITE_ACTIVATED IS_MULTISITE_ACTIVATED} property as
     * defined by this proxy.
     * 
     * @return <b>true</b> if the database has been activated for multisite
     *         operations, even if the database is the only existing replica;
     *         <b>false</b> if not activated.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_MULTISITE_ACTIVATED IS_MULTISITE_ACTIVATED}
     *             property.
     */
    boolean getIsMultisiteActivated() throws WvcmException;

    /**
     * The highest feature level supported by the server.
     */
    PropertyName<Long> MAX_COMPATIBLE_FEATURE_LEVEL =
        new PropertyName<Long>(PROPERTY_NAMESPACE,
                               "max-compatible-feature-level");

    /**
     * Returns the value of the
     * {@link #MAX_COMPATIBLE_FEATURE_LEVEL MAX_COMPATIBLE_FEATURE_LEVEL}
     * property as defined by this proxy.
     * 
     * @return A long value designating the highest feature level supported by
     *         the server.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #MAX_COMPATIBLE_FEATURE_LEVEL MAX_COMPATIBLE_FEATURE_LEVEL}
     *             property.
     */
    long getMaxCompatibleFeatureLevel() throws WvcmException;

    /**
     * The lowest feature level supported by the server
     */
    PropertyName<Long> MIN_COMPATIBLE_FEATURE_LEVEL =
        new PropertyName<Long>(PROPERTY_NAMESPACE,
                               "min-compatible-feature-level");

    /**
     * Returns the value of the
     * {@link #MIN_COMPATIBLE_FEATURE_LEVEL MIN_COMPATIBLE_FEATURE_LEVEL}
     * property as defined by this proxy.
     * 
     * @return A long value designating the lowest feature level supported by
     *         the server.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #MIN_COMPATIBLE_FEATURE_LEVEL MIN_COMPATIBLE_FEATURE_LEVEL}
     *             property.
     */
    long getMinCompatibleFeatureLevel() throws WvcmException;

    /**
     * An extension of the standard PropertyName that may be used by clients to
     * access named values on the ClearQuest server as CM API properties of this
     * database.
     */
    public class NamedValueName extends PropertyName<String>
    {
        /**
         * Constructs a PropertyName that denotes the name component of a
         * ClearQuest session name/value pair. A PropertyRequest with an
         * instance of this class requests the value associated with the given
         * name from ClearQuest. Writing a value to the server using an instance
         * of this PropertyName will cause the value to be associated with the
         * name during this session. The value is always a String and is
         * accessible only to this proxy's CqProvider or to hooks fired by this
         * proxy's CqProvider.
         * 
         * @param name The name that is to be associated with a value in the
         *            ClearQuest session.
         */
        public NamedValueName(String name)
        {
            super(StpExBase.NAMED_VALUE_NAMESPACE, name);
        }
    }

    /**
     * Retrieves the value of a previously requested property named by an
     * instance of NamedValueName having the same parameters as this method.
     * This is just shorthand for
     * <code>getProperty(new NamedValueName(name))</code>
     * 
     * @param name The name for which a value was requested.
     * @return A String containing the value of the NameValue property
     * @throws WvcmException If the NameValue for the given name was not
     *             requested or otherwise not available.
     */
    String getNamedValue(String name) throws WvcmException;

    /**
     * A defines a new value for a NamedValue property of a given name. The
     * property is not updated until the next "do" operations is invoked.
     * Shorthand for <code>setProperty(new NamedValueName(name), value)</code>
     * 
     * @param name The name of the NamedValue whose value is to be set.
     * @param value The new value for the NameValue property identified by the
     *            name parameter.
     */
    void setNamedValue(String name,
                       String value);

    /**
     * A list of all the name/value properties defined for this database
     * session. This is a pseudo property, similar in concept to
     * {@link javax.wvcm.Resource#ALL_CUSTOM_PROPERTIES}. When it appears in a
     * PropertyRequest, the server replies by populating the result proxy with
     * all named values defined for the database session. The value associated
     * with a given name can then be retrieved from the proxy using a
     * NamedValueName for the name. All of the returned named value properties
     * can be retrieved as an StpProperty.List using the getAllNameValues method
     * of this resource.
     */
    PropertyName<StpProperty.List<StpProperty<String>>> ALL_NAMED_VALUES =
        new PropertyName<StpProperty.List<StpProperty<String>>>(PROPERTY_NAMESPACE,
                                                                "all-named-values");

    /**
     * Returns the value of the {@link #ALL_NAMED_VALUES ALL_NAMED_VALUES}
     * property as defined by this proxy.
     * 
     * @return A List of StpProperty&lt;String&gt; objects, each representing a
     *         named value defined for this database session.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_NAMED_VALUES ALL_NAMED_VALUES} property.
     */
    StpProperty.List<StpProperty<String>> getAllNamedValues()
        throws WvcmException;

    /**
     * Defines a new value for the {@link #ALL_NAMED_VALUES ALL_NAMED_VALUES}
     * property of this proxy.
     * 
     * @param allNameValues An StpProperty.List associating names to String
     *            values. The entries of this list <b>replace</b> the current
     *            entries in the name/value map accessible to scripts on the
     *            server. If this property is empty or <b>null</b>, the server
     *            map will be emptied. To update individual name/value
     *            associations, use setProperty with a NamedValueName and string
     *            value.
     */
    void setAllNamedValues(StpProperty.List<StpProperty<String>> allNameValues);

    /**
     * The groups explicitly subscribed to this database. It does not include
     * the groups implicitly subscribed to this database because their
     * {@link CqGroup#IS_SUBSCRIBED_TO_ALL_DATABASES} property is <b>true</b>.
     */
    PropertyName<ResourceList<CqGroup>> SUBSCRIBED_GROUPS =
        new PropertyName<ResourceList<CqGroup>>(PROPERTY_NAMESPACE,
                                                "subscribed-groups");

    /**
     * Returns the value of the {@link #SUBSCRIBED_GROUPS SUBSCRIBED_GROUPS}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a CqGroup proxy for each group
     *         explicitly subscribed to this user database. Will never be
     *         <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SUBSCRIBED_GROUPS SUBSCRIBED_GROUPS} property.
     */
    ResourceList<CqGroup> getSubscribedGroups() throws WvcmException;

    /**
     * The users explicitly subscribed to this database. It does not include the
     * users implicitly subscribed to this database because their
     * {@link CqUser#IS_SUBSCRIBED_TO_ALL_DATABASES} property is <b>true</b>.
     */
    PropertyName<ResourceList<CqUser>> SUBSCRIBED_USERS =
        new PropertyName<ResourceList<CqUser>>(PROPERTY_NAMESPACE,
                                               "subscribed-users");

    /**
     * Returns the value of the {@link #SUBSCRIBED_USERS SUBSCRIBED_USERS}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a CqUser proxy for each group
     *         explicitly subscribed to this user database. Will never be
     *         <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SUBSCRIBED_USERS SUBSCRIBED_USERS} property.
     */
    ResourceList<CqUser> getSubscribedUsers() throws WvcmException;

    /**
     * Answers whether or not the package supporting the integration of
     * ClearQuest and ReqiuisitePro has been enabled for this user database
     */
    static PropertyName<Boolean> IS_REQUISITE_PRO_ENABLED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-requisite-pro-enabled");

    /**
     * Returns the value of the
     * {@link #IS_REQUISITE_PRO_ENABLED IS_REQUISITE_PRO_ENABLED} property as
     * defined by this proxy.
     * 
     * @return <b>true</b> if the RequisitePro package has been enabled on this
     *         database; <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_REQUISITE_PRO_ENABLED IS_REQUISITE_PRO_ENABLED}
     *             property.
     */
    boolean getIsRequisiteProEnabled() throws WvcmException;

    /**
     * Answers whether or not full-text search has been enabled for this user
     * database.
     */
    static PropertyName<Boolean> IS_FULL_TEXT_SEARCH_ENABLED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-full-text-search-enabled");

    /**
     * Returns the value of the
     * {@link #IS_FULL_TEXT_SEARCH_ENABLED IS_FULL_TEXT_SEARCH_ENABLED} property
     * as defined by this proxy.
     * 
     * @return <b>true</b> if full-text search has been enabled on this
     *         database; <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_FULL_TEXT_SEARCH_ENABLED IS_FULL_TEXT_SEARCH_ENABLED}
     *             property.
     */
    boolean getIsFullTextSearchEnabled() throws WvcmException;

    /**
     * Answers whether or not this database, using the CqProvider of this proxy,
     * provides support for reports, including access to CqReport and
     * CqReportFormat resources and the generation of reports via
     * CqReport.doMakeReport().
     */
    static PropertyName<Boolean> IS_REPORTING_ENABLED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-reporting-enabled");

    /**
     * Returns the value of the
     * {@link #IS_REPORTING_ENABLED IS_REPORTING_ENABLED} property as defined by
     * this proxy.
     * 
     * @return <b>true</b> if CqReport and CqReportFormat resources are
     *         supported by this database through this proxy's CqProvider;
     *         <b>false</b> otherwise.
     * 
     * @throws WvcmException
     *             if this proxy does not define a value for the
     *             {@link #IS_REPORTING_ENABLED IS_REPORTING_ENABLED} property.
     */
    boolean getIsReportingEnabled() throws WvcmException;

    /**
     * Answers whether or not this database, using the CqProvider of this proxy,
     * provides support for the generation of charts via CqQuery.doMakeChart().
     */
    static PropertyName<Boolean> IS_CHARTING_ENABLED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-charting-enabled");

    /**
     * Returns the value of the {@link #IS_CHARTING_ENABLED IS_CHARTING_ENABLED}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if charting is supported by this database through
     *         this proxy's CqProvider; <b>false</b> otherwise.
     * 
     * @throws WvcmException
     *             if this proxy does not define a value for the
     *             {@link #IS_CHARTING_ENABLED IS_CHARTING_ENABLED} property.
     */
    boolean getIsChartingEnabled() throws WvcmException;

    /**
     * Answers whether or not this is a test database
     */
    static PropertyName<Boolean> IS_TEST_DATABASE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-test-database");

    /**
     * Returns the value of the {@link #IS_TEST_DATABASE IS_TEST_DATABASE}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if this user database has been designated a test
     *         database; <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_TEST_DATABASE IS_TEST_DATABASE}
     *             property.
     */
    boolean getIsTestDatabase() throws WvcmException;

    /**
     * A list of the record types that are being indexed for full-text searches.
     */
    static PropertyName<ResourceList<CqRecordType>> INDEXED_RECORD_TYPES =
        new PropertyName<ResourceList<CqRecordType>>(PROPERTY_NAMESPACE,
                                                     "indexed-record-types");

    /**
     * Returns the value of the
     * {@link #INDEXED_RECORD_TYPES INDEXED_RECORD_TYPES} property as defined by
     * this proxy.
     * 
     * @return A ResourceList containing a CqRecordType proxy for any record
     *         type that is being indexed for use in full-text searches.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #INDEXED_RECORD_TYPES INDEXED_RECORD_TYPES} property.
     */
    ResourceList<CqRecordType> getIndexedRecordTypes() throws WvcmException;

    /**
     * A list of all report formats defined in this database and accessible to
     * the current user.
     */
    PropertyName<ResourceList<CqReportFormat>> REPORT_FORMATS =
        new PropertyName<ResourceList<CqReportFormat>>(PROPERTY_NAMESPACE,
                                                       "report-formats");

    /**
     * Returns the value of the {@link #REPORT_FORMATS REPORT_FORMATS} property
     * as defined by this proxy.
     * 
     * @return A ResourceList containing a CqReportFormat proxy for each report
     *         format defined in the public and personal query folders . Will
     *         never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #REPORT_FORMATS REPORT_FORMATS} property.
     */
    ResourceList<CqReportFormat> getReportFormats() throws WvcmException;

    /**
     * Returns a CqUserInfo structure containing read-only information about the
     * user currently logged into this database.
     */
    PropertyName<CqUserInfo> USER_INFO = 
        new PropertyName<CqUserInfo>(PROPERTY_NAMESPACE, "user-info");
    
    /**
     * Returns the value of the {@link #USER_INFO USER_INFO} property as defined
     * by this proxy.
     * 
     * @return A CqUserInfo instance containing read-only information about the
     *         user currently logged into this database. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #USER_INFO USER_INFO} property.
     */
    CqUserInfo getUserInfo() throws WvcmException;

    /**
     * Removes all modified and moribund resources from the change context of
     * this database; thereby reverting those resources to their previous,
     * unmodified or undeleted state.
     * 
     * @param feedback A Feedback object requesting properties of this database
     *            and/or properties of the reverted resources to be returned
     *            from the server after successful completion of the operation.
     * 
     * @throws WvcmException If the resources cannot be reverted.
     */
    CqUserDb doClearContext(Feedback feedback) throws WvcmException;

    /**
     * Selectively delivers (commits) modified resources to the database and/or
     * destroys moribund resources from the database and removes them from the
     * change context.
     * <p>
     * NOTE: A ClearQuest query created and delivered in one CqProvider instance
     * may not appear in another already-existing CqProvider instance. To make
     * them available, create a new CqProvider instance <b>after</b> the
     * delivery is complete.
     * <p>
     * Known places in the implementation that will have this problem include:<br>
     * Creation or deletion of a ClearQuest query (modification is OK).<br>
     * Creation or deletion of a ClearQuest CqQueryFolder.<br>
     * Modification of a ClearQuest CqDynamicChoiceList (creation is not
     * supported anyway).
     * 
     * @param feedback A request for specific properties from the resources that
     *            were directly or indirectly impacted by this operation, which,
     *            in this case would be the modified resources that were
     *            delivered by this operation. May be <b>null</b>; in which
     *            case this information will not be requested from the server.
     * @param deliveryOrder The modified and moribund resources currently in
     *            this change context that are to be delivered or deleted. They
     *            will be delivered or deleted in the order specified by the
     *            proxies in the given list. To deliver/delete all
     *            modified/moribund resources in an arbitrary order, use
     *            {@link CqProvider#DELIVER_ALL}. Must not be <b>null</b> nor
     *            any of the other special delivery order constants AUTO, HOLD,
     *            or DELIVER.
     * 
     * @throws WvcmException if any preconditions are not satisfied or if other
     *             errors are detected while writing the records to the
     *             database.
     */

    CqUserDb doDeliver(Feedback feedback,
                       List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * Selectively reverts modified or moribund resources in this database's
     * change context to their previous, unmodified/undeleted state. To revert
     * all resources in the change context, in an arbitrary order, use
     * {@link #doClearContext(Feedback)}. This operation is semantically
     * equivalent to invoking doRevert() on each resource in the given list, but
     * will be more efficient because there is only one round-trip to the server
     * required.
     * <p>
     * Note that doRevert removes all changes to the resource from the change
     * context. It is not an undo operation, but a revert. If multiple changes
     * have been made to a resource (since the last time it was delivered) they
     * will all be discarded by doRevert.
     * 
     * @param feedback A request for specific properties from the resources that
     *            were directly or indirectly impacted by this operation, which,
     *            in this case would be the modified/moribund resources that
     *            were reverted by this operation. May be <b>null</b>; in which
     *            case this information will not be requested from the server.
     * @param revertOrder The modified/moribund resources currently in this
     *            change context to be reverted to their original state. They
     *            will be reverted/undeleted in the order specified by the
     *            proxies in this argument. Must not be <b>null</b> nor any of
     *            the special delivery order constants AUTO, HOLD, DELIVER, or
     *            DELIVER_ALL.
     * 
     * @throws WvcmException if any preconditions are not satisfied or if other
     *             errors are detected while writing the records to the
     *             database.
     */

    CqUserDb doRevert(Feedback feedback,
                      List<CqContextResource> revertOrder) throws WvcmException;

    /**
     * Searches the database for records having a given display name.
     * 
     * @param displayName A String containing the display name of the record to
     *            be found.
     * 
     * @param possibleRecordTypes An array of locations containing a
     *            CqRecordType location for each type of record to be considered
     *            in the search. May not contain record type family names. May
     *            be <b>null</b> or empty to force consideration of all record
     *            types.
     *            <p>
     *            The record type array is used to control the search for the
     *            record identified by the displayName argument. This list of
     *            record types is iterated and processed in the order given
     *            until a record is found. If any of the record type locations
     *            provided in this argument is invalid (such as referring to a
     *            record type family), an exception is thrown that identifies
     *            the invalid location. If a null or empty array value is
     *            provided, all record types defined in the schema are searched,
     *            with the search order being from the most frequently found
     *            record type searched first to the least found record type
     *            being searched last.
     * 
     * @param mode A FindMode enumerator identifying the type of record to
     *            search for.
     * 
     * @param feedback A Feedback object in which may be requested search
     *            progress reports and additional properties to be included in
     *            the result proxy. May be <b>null</b> if no additional
     *            feedback is needed.
     * 
     * @return A CqRecord proxy representing a record of one of the requested
     *         types or type categories whose display name matches the given
     *         displayName argument; <b>null</b> if no such record can be
     *         found. If a record is found, the returned proxy will be populated
     *         with the properties requested by the feedback parameter.
     * 
     * @throws WvcmException If the search as specified could not be completed;
     *             for example, if the database or the specified record types
     *             are not accessible. If all record types can be searched as
     *             specified, failure to find a matching record <i>does not</i>
     *             cause an exception but results in a null result.
     */
    CqRecord doFindRecord(String displayName,
                          FindMode mode,
                          Feedback feedback,
                          Location... possibleRecordTypes) throws WvcmException;

    /**
     * Searches the database for records having a given dbid.
     * 
     * @param dbid The dbid of the record to be found.
     * 
     * @param mode A FindMode enumerator identifying the type of record to
     *            search for.
     * 
     * @param possibleRecordTypes An array of locations containing a
     *            CqRecordType location for each type of record to be considered
     *            in the search. May not contain record type family names. May
     *            be <b>null</b> or empty to force consideration of all record
     *            types.
     *            <p>
     *            The record type array is used to control the search for the
     *            record identified by the displayName argument. This list of
     *            record types is iterated and processed in the order given
     *            until a record is found. If any of the record type locations
     *            provided in this argument is invalid (such as referring to a
     *            record type family), an exception is thrown that identifies
     *            the invalid location. If a null or empty array value is
     *            provided, all record types defined in the schema are searched,
     *            with the search order being from the most frequently found
     *            record type searched first to the least found record type
     *            being searched last.
     * 
     * @param feedback A Feedback object in which may be requested search
     *            progress reports and additional properties to be included in
     *            the result proxy. May be <b>null</b> if no additional
     *            feedback is needed.
     * 
     * @return A CqRecord proxy representing a record of one of the requested
     *         types or type categories whose dbid name matches the given
     *         dbid argument; <b>null</b> if no such record can be
     *         found. If a record is found, the returned proxy will be populated
     *         with the properties requested by the feedback parameter.
     * 
     * @throws WvcmException If the search as specified could not be completed;
     *             for example, if the database or the specified record types
     *             are not accessible. If all record types can be searched as
     *             specified, failure to find a matching record <i>does not</i>
     *             cause an exception but results in a null result.

     * @see #doFindRecord(long, com.ibm.rational.wvcm.stp.cq.CqUserDb.FindMode,
     *      Feedback, Location[])
     */
    CqRecord doFindRecord(long dbid,
                          FindMode mode,
                          Feedback feedback,
                          Location... possibleRecordTypes) throws WvcmException;

    /**
     * Enumerates the options for restricting the finding of records based on
     * their record class.
     */
    enum FindMode implements StpExEnumeration
    {
        FIND_ANY_RECORD, FIND_STATELESS_RECORD, FIND_STATEFUL_RECORD;
    }

    /**
     * Returns a list of all the resources in the database whose names differ
     * from a given name only in their site extensions. Currently, only query
     * folder items and records of a stateless record type could possibly need
     * site extensions to disambiguate them, so applying this method to any
     * other resource is simply a waste of time and is likely to raise an
     * exception.
     * <p>
     * Applying this method to a stateless record type location has a special
     * meaning. In that case <i>all</i> instances of that record type that
     * require site extension are returned.
     * 
     * @param name A (potentially ambiguous) user-friendly location
     *            specification for a ClearQuest record, record type, or query
     *            folder item, whose repo field, if present, must name this
     *            database.
     * 
     * @param feedback A Feedback object in which may be requested search
     *            progress reports and additional properties to be included in
     *            the result proxies. May be <b>null</b> if no additional
     *            feedback is needed.
     * 
     * @return A ResourceList containing a proxy for each resource in the
     *         database whose site-extended name differs from the given name
     *         only in the site extensions used. Will never be <b>null</b> or
     *         empty.
     * 
     * @throws WvcmException If the database is not accessible, if the given
     *             name identifies a resource other than a record or query
     *             folder item in this database, or if the given name cannot be
     *             extended in any way to name any resource.
     */
    ResourceList<CqResource> doFindSiteExtendedNames(StpLocation name,
                                                     Feedback feedback)
        throws WvcmException;

    /**
     * Upgrades this database's user and group information to match that of its
     * db-set. This method copies the changes from the db-set to the user
     * database. You should call this function to upgrade a database after
     * making changes to the users and groups of the db-set.
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return A proxy for this user database (after the upgrade is complete),
     *         whose properties are specified by feedback.
     * @throws WvcmException
     */
    CqUserDb doUpgradeUsersAndGroups(Feedback feedback) throws WvcmException;

    /**
     * Verifies that a CqQueryFolderItem of a given type can be created at a
     * given location. The operation checks that the characters used in the
     * location are valid for the target database and that no resource currently
     * exists at the specified location.
     * 
     * @param item A proxy for the item whose location is to be tested.
     * 
     * @param feedback A Feedback object that specifies the properties to be
     *            included in the result proxy (if a resource exists at the
     *            specified location). If the argument is null, the properties
     *            defined by the result proxy are unspecified.
     * 
     * @return If the location is valid and there is no resource currently at
     *         the specified location, the result is <b>null</b>. If the
     *         location is valid, but a resource already exists at that
     *         location, a proxy for that resource is returned, populated with
     *         any properties requested by the feedback argument.
     * 
     * @throws WvcmException if the location is not valid on the target
     *             database.
     */
    CqQueryFolderItem doValidateLocation(CqQueryFolderItem item,
                                         Feedback feedback)
        throws WvcmException;

    /**
     * The interface for an API object that may be used to define a filter for a
     * full-text search of the database. The search key specifies an attribute
     * of the database object whose value will be used to select the object for
     * inclusion in the result set of the full-text search. In the SearchFilter,
     * each SearchKey is associated with a target value that specifies what
     * attribute values will cause the object to be selected.
     * <p>
     * The currently defined SearchKey types are
     * <ul>
     * <li>{@link SearchLevel}: (SearchKey&lt;String>) selects an object
     * having any field matching the specification in the target String. The
     * SearchMode enumerator specifies the type of specification to be found in
     * the target string. Currently there is only one SearchMode, SIMPLE, which
     * represents a full-text search using a simple specification.
     * <li>{@link WordMode}: (SearchKey&lt;List&lt;String>>) selects an object
     * having any field matching the words in the target list. The WordMode
     * enumerator specifies how the target words must match a field to select
     * the object.
     * <li>{@link DateLimit}: (SearchKey&lt;Date>) selects an object based on
     * the relationship between the object's creation or modification date and
     * the target date. The DateLimit enumerator specifies which date attribute
     * to compare against and the relationship with the target date that will
     * select the object.
     * <li>{@link CqRecordType}: (SearchKey&lt;Boolean>) selects a record
     * based on its record type. If the target value is Boolean.TRUE, a record
     * whose record type is the same as the record type denoted by the
     * CqRecordType object will be selected. If several CqRecordType keys (with
     * TRUE target values) are present in a SearchFilter, a record having anyone
     * of those record types is selected.
     * <li>{@link CqFieldDefinition}: (SearchKey&lt;List&lt;String>>) selects
     * a record having a specific field whose value matches all of the target
     * strings. The CqFieldDefinition specifies the record type and the field of
     * that record type that must match the target strings. Each target string
     * is a simple search specification.
     * </ul>
     * 
     * @param <T> The type of object used to specify the value or values that
     *            will select or reject an object based on the value it has for
     *            the key attribute.
     */
    public static interface SearchKey<T>
    {}

    /**
     * WordMode SearchKeys limit the search results to just those objects that
     * contain the targeted words associated with the WordMode key in the
     * filter.
     */
    public static enum WordMode implements SearchKey<List<String>>
    {
        /** Match all of the words in the target list */
        ALL,
        /** Match all of the words in the target list in the order given */
        EXACT,
        /** Match any (at least one) of the words in the target list */
        ANY,
        /** Match none (do not match any) of the words in the target list */
        NONE
    }

    /**
     * A DateLimit SearchKey limits the search to just those objects whose
     * creation or modification time are in a target range.
     */
    public static enum DateLimit implements SearchKey<Date>
    {
        /** Matches an object created on or after the target date */
        CREATED_AFTER,

        /** Matches an object created on or before the target date */
        CREATED_BEFORE,

        /** Matches an object modified on or after the target date */
        MODIFIED_AFTER,

        /** Matches an object modified on or before the target date */
        MODIFIED_BEFORE
    }

    /** Specifies a simple search string */
    public static enum SearchLevel implements SearchKey<String>
    {
        /** Interpret the associated argument as a simple search string and
         * match accordingly against that.
         */
        SIMPLE
    }

    /**
     * A collection of SearchKey/target-value pairs that collectively define the
     * filter for selecting objects for inclusion in a full-text search. In
     * general, an object must match all of the given keys in the filter to be
     * added to the result set. The only exception to this rule are the
     * CqRecordType keys. If more than one CqRecordType key is present, the
     * object needs only match one of them to be a candidate for the result
     * set--the object would need to match all the other types of keys to
     * actually be selected.
     */
    public static interface SearchFilter
    {
        /**
         * Adds a SearchKey and its target value to the full text search filter.
         * Each SearchKey specifies what attribute of an object is to be
         * filtered on and the target value specifies the value against which
         * the attribute value will be compared for possible inclusion in the
         * result set.
         * 
         * @param <U> The data type used to specify the target value. This type
         *            must match the type parameter of the key used and so
         *            currently this must be String, List&lt;String>, Date, or
         *            Boolean.
         * @param key A SearchKey&lt;U> object representing the attribute of
         *            each candidate object that the search should filter on.
         *            Currently this may be a SearchLevel enumerator, a
         *            DateLimit enumerator, a WordMode enumerator, a
         *            CqRecordType proxy, or a CqFieldDefinition proxy.
         * @param targetValue An instance of class U that specifies which values
         *            of the key attribute will select or reject a record in the
         *            search. If this value is <b>null</b> (or Boolean.FALSE)
         *            the corresponding SearchKey will be removed from the
         *            search specification.
         * @return The value previously associated with the SearchKey.
         */
        <U> U put(SearchKey<U> key,
                  U targetValue);

        /**
         * Extends the target value list associated with a SearchKey by a single
         * item.
         * 
         * @param <U> The type of item in a target value list; currently must be
         *            String.
         * @param key The SearchKey whose target value list is to be extended.
         *            The SeachKey does not need to already be in the
         *            SearchFilter. Currently the SearchKey must be a WordMode
         *            enumerator or a CqFieldDefinition proxy.
         * @param targetItem The new item to be added to the target value list
         * @return The target value list after modification.
         */
        <U> List<U> add(SearchKey<List<U>> key,
                        U targetItem);

        /**
         * Returns the target value currently associated with a given SearchKey
         * 
         * @param <U> The data type used to specify the value against which the
         *            key attribute will be compared. This type must match the
         *            type parameter of the key used and so currently this must
         *            be String, List&lt;String>, Date, or Boolean.
         * @param key A SearchKey<U> object representing the attribute of each
         *            candidate record that the search should filter on.
         *            Currently this may be a SearchLevel enumerator, a
         *            DateLimit enumerator, a WordMode enumerator, a
         *            CqRecordType proxy, or a CqFieldDefinition proxy.
         * @return The value currently associated with the given key in this
         *         search filter; <b>null</b> if no association is currently
         *         defined.
         */
        <U> U get(SearchKey<U> key);

        /**
         * @return A List of all the SearchKey/target-values pairs defined by
         *         this SearchFilter.
         */
        List<Map.Entry<SearchKey<Object>, Object>> getAll();

        /**
         * Retrieves SearchKey/target-value pairs of a given target-value type
         * 
         * @param <U> The data type used to specify the value against which the
         *            key attribute will be compared. This type must match the
         *            type parameter of a defined SearchKey and so currently
         *            this must be String, List&lt;String>, Date, or Boolean.
         * @param type The Class object for the desired target-value data type.
         * @return All of the entries in this SearchFilter whose target value
         *         type matches the given type. The result will never be <b>null</b>,
         *         but the returned list may be empty.
         */
        <U> List<Map.Entry<SearchKey<U>, U>> getEntries(Class<U> type);

        /**
         * Retrieves SearchKey/target-value pairs by SearchKey type
         * 
         * @param <U> The data type used to specify the value against which the
         *            key attribute will be compared. This type must match the
         *            type parameter of a defined SearchKey and so currently
         *            this must be String, List&lt;String>, Date, or Boolean.
         * @param <V> A subclass of SearchKey<U>
         * @param keyType The type of keys to retrieve from this search
         *            specification
         * @return All entries of this SearchFilter whose key is of the
         *         specified type. The result will never be null, but the
         *         returned list may be empty.
         */
        <V extends SearchKey<U>, U> List<Map.Entry<V, U>> getEntriesByKey(Class<V> keyType);

        /**
         * @return The SearchLevel entry of this SearchFilter; otherwise null
         */
        Map.Entry<SearchLevel, String> getStringFilter();

        /**
         * @return A list of the CqRecordType entries of this SearchFilter; will
         *         never be <b>null</b> but the list may be empty.
         */
        List<Map.Entry<CqRecordType, Boolean>> getRecordTypeFilters();

        /**
         * @return A list of the CqFieldDefinition entries of this SearchFilter;
         *         will never be <b>null</b> but the list may be empty.
         */
        List<Map.Entry<CqFieldDefinition, List<String>>> getFieldFilters();

        /**
         * @return A list of the WordMode entries of this SearchFilter; will
         *         never be <b>null</b> but the list may be empty.
         */
        List<Map.Entry<WordMode, List<String>>> getWordFilters();

        /**
         * @return A list of the DateLimit entries of this SearchFilter; will
         *         never be <b>null</b> but the list may be empty.
         */
        List<Map.Entry<DateLimit, Date>> getDateFilters();

        /**
         * Removes all SearchKey entries from this SearchFilter
         */
        void clear();
    }

    /**
     * Constructs a SearchFilter instance with an initial simple specification
     * string.
     * 
     * @param simpleString A search specification using the simple specification
     *            syntax. If <b>null</b>, an empty SearchFilter will be
     *            returned.
     * @return A SearchFilter instance defining the simple search specification
     *         derived from the simpleString argument.
     */
    SearchFilter buildSearchFilter(String simpleString);

    /**
     * Searches the indexed content of the database for objects that match a
     * given filter. Information about each selected object is returned as a
     * packet of data called a <i>hit</i>. The matching process assigns a score
     * to the hit between 1 and 100, with 100 designating a near-perfect match.
     * The hits are ranked according to this score and are returned in order
     * highest rank to lowest.
     * <p>
     * When this method returns, the highest ranking hits will be available
     * immediately for inspection on the client. An argument to this method
     * controls the size of this first set of hits. If there were more hits
     * found than could be returned immediately, the remaining hits can be
     * requested using the CqHitSet.doGetMore() method.
     * 
     * @param searchFilter A SearchFilter specifying which objects of the
     *            database are to be selected for inclusion in the result set.
     * @param setParams 0, 1, or 2 long values specifying how the hits are to be
     *            returned from the server to the client.
     *            <ul>
     *            <li>setParams[0]: The maximum number of hits the client will
     *            accept from the server in the immediate response to this
     *            request. Based on the total number of hits found and the load
     *            on the server, however, the actual number of hits returned may
     *            be less than this number. If this value is zero (or omitted)
     *            the number of hits returned will be determined solely by the
     *            server. In which case, the client should be prepared to accept
     *            all hits found by the search.
     *            <li>setParams[1]: The index of the first hit to be returned
     *            to the client, where 1 is the index of the first hit found by
     *            the search. If this value is zero (or omitted) the first hit
     *            found will be the first hit returned.
     *            </ul>
     * @return An iterator over the hits found in the search.
     * 
     * @see CqHitSet#doGetMore(long)
     */
    CqHitSet doFullTextSearch(SearchFilter searchFilter,
                              long... setParams) throws WvcmException;
}
